@@ -22,10 +22,11 @@
 
 #include <config.h>
 #include <string.h>
-#include <glib/gi18n.h>
-#include <gtk/gtk.h>
+#include "gtk-utils.h"
 #include <libgnome/gnome-help.h>
 
+#define LOAD_BUFFER_SIZE 65536
+
 
 static void
 count_selected (GtkTreeModel *model,
@@ -66,7 +67,9 @@ _gtk_message_dialog_new (GtkWindow        *parent,
 	va_list       args;
 	const gchar  *text;
 	int           response_id;
-	char         *escaped_message, *markup_text;
+	char         *markup_text;
+
+	g_return_val_if_fail ((message != NULL) || (secondary_message != NULL), NULL);
 
 	if (stock_id == NULL)
 		stock_id = GTK_STOCK_DIALOG_INFO;
@@ -85,18 +88,26 @@ _gtk_message_dialog_new (GtkWindow        *parent,
 
 	label = gtk_label_new ("");
 
-	escaped_message = g_markup_escape_text (message, -1);
-	if (secondary_message != NULL) {
-		char *escaped_secondary_message = g_markup_escape_text (secondary_message, -1);
-		markup_text = g_strdup_printf ("<span weight=\"bold\" size=\"larger\">%s</span>\n\n%s",
-					       escaped_message,
-					       escaped_secondary_message);
-		g_free (escaped_secondary_message);
-	} else
-		markup_text = g_strdup (escaped_message);
+	if (message != NULL) {
+		char *escaped_message;
+
+		escaped_message = g_markup_escape_text (message, -1);
+		if (secondary_message != NULL) {
+			char *escaped_secondary_message = g_markup_escape_text (secondary_message, -1);
+			markup_text = g_strdup_printf ("<span weight=\"bold\" size=\"larger\">%s</span>\n\n%s",
+						       escaped_message,
+						       escaped_secondary_message);
+			g_free (escaped_secondary_message);
+		}
+		else
+			markup_text = g_strdup (escaped_message);
+		g_free (escaped_message);
+	}
+	else
+		markup_text = g_markup_escape_text (secondary_message, -1);
+
 	gtk_label_set_markup (GTK_LABEL (label), markup_text);
 	g_free (markup_text);
-	g_free (escaped_message);
 
 	gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);
 	gtk_label_set_selectable (GTK_LABEL (label), TRUE);
@@ -257,7 +268,7 @@ _gtk_request_dialog_run (GtkWindow        *parent,
 
 	button = create_button (GTK_STOCK_CANCEL, no_button_text);
 	gtk_dialog_add_action_widget (GTK_DIALOG (dialog),
-				      button, 
+				      button,
 				      GTK_RESPONSE_CANCEL);
 
 	button = create_button (GTK_STOCK_OK, yes_button_text);
@@ -332,14 +343,14 @@ _gtk_yesno_dialog_new (GtkWindow        *parent,
 
 	button = create_button (GTK_STOCK_CANCEL, no_button_text);
 	gtk_dialog_add_action_widget (GTK_DIALOG (d),
-				      button, 
+				      button,
 				      GTK_RESPONSE_CANCEL);
 
 	/**/
 
 	button = create_button (GTK_STOCK_OK, yes_button_text);
 	gtk_dialog_add_action_widget (GTK_DIALOG (d),
-				      button, 
+				      button,
 				      GTK_RESPONSE_YES);
 
 	/**/
@@ -350,16 +361,6 @@ _gtk_yesno_dialog_new (GtkWindow        *parent,
 }
 
 
-static void
-toggle_visibility (GtkWidget *widget)
-{
-	if (GTK_WIDGET_VISIBLE (widget))
-		gtk_widget_hide (widget);
-	else
-		gtk_widget_show (widget);
-}
-
-
 GtkWidget*
 _gtk_error_dialog_new (GtkWindow        *parent,
 		       GtkDialogFlags    flags,
@@ -375,7 +376,7 @@ _gtk_error_dialog_new (GtkWindow        *parent,
 	GtkWidget     *vbox;
 	GtkWidget     *text_view;
 	GtkWidget     *scrolled = NULL;
-	GtkWidget     *button;
+	GtkWidget     *expander;
 	GtkTextBuffer *text_buf;
 	GtkTextIter    iter;
 	char          *stock_id;
@@ -387,7 +388,7 @@ _gtk_error_dialog_new (GtkWindow        *parent,
 	stock_id = GTK_STOCK_DIALOG_ERROR;
 
 	dialog = gtk_dialog_new_with_buttons ("",
-					      parent, 
+					      parent,
 					      flags,
 					      GTK_STOCK_OK, GTK_RESPONSE_OK,
 					      NULL);
@@ -420,7 +421,7 @@ _gtk_error_dialog_new (GtkWindow        *parent,
 		va_end (args);
 		escaped_secondary_message = g_markup_escape_text (secondary_message, -1);
 
-		markup_text = g_strdup_printf ("<span weight=\"bold\" size=\"larger\">%s</span>\n\n%s",
+		markup_text = g_strdup_printf ("<span weight=\"bold\" size=\"larger\">%s</span>\n\n%s\n",
 					       escaped_message,
 					       escaped_secondary_message);
 
@@ -434,9 +435,9 @@ _gtk_error_dialog_new (GtkWindow        *parent,
 	g_free (escaped_message);
 
 	if (view_output) {
-		/* Button */
+		/* Expander */
 
-		button = gtk_toggle_button_new_with_mnemonic (_("Command _Line Output"));
+		expander = gtk_expander_new_with_mnemonic (_("Command _Line Output"));
 
 		/* Add text */
 
@@ -476,40 +477,22 @@ _gtk_error_dialog_new (GtkWindow        *parent,
 	vbox = gtk_vbox_new (FALSE, 6);
 	gtk_container_set_border_width (GTK_CONTAINER (vbox), 5);
 
-	hbox = gtk_hbox_new (FALSE, 6);
-	gtk_box_pack_start (GTK_BOX (hbox), image,
-			    FALSE, FALSE, 0);
-	gtk_box_pack_start (GTK_BOX (hbox), label,
-			    FALSE, FALSE, 0);
-	gtk_box_pack_start (GTK_BOX (vbox), hbox,
-			    TRUE, TRUE, 0);
+	hbox = gtk_hbox_new (FALSE, 12);
+	gtk_box_pack_start (GTK_BOX (hbox), image, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (vbox), hbox, TRUE, TRUE, 0);
 
 	if (view_output) {
-		hbox = gtk_hbox_new (FALSE, 6);
-		gtk_box_pack_start (GTK_BOX (hbox), button,
-			    FALSE, FALSE, 0);
-		gtk_box_pack_start (GTK_BOX (vbox), hbox,
-				    TRUE, TRUE, 0);
-
 		gtk_container_add (GTK_CONTAINER (scrolled), text_view);
-		gtk_box_pack_start (GTK_BOX (vbox), scrolled,
-				    FALSE, FALSE, 0);
-
-		g_signal_connect_swapped (G_OBJECT (button),
-					  "clicked", 
-					  G_CALLBACK (toggle_visibility), 
-					  scrolled);				    
+		gtk_container_add (GTK_CONTAINER (expander), scrolled);
+		gtk_box_pack_start (GTK_BOX (vbox), expander, TRUE, TRUE, 0);
 	}
 
 	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox),
 			    vbox,
 			    FALSE, FALSE, 0);
-
 	gtk_widget_show_all (vbox);
 
-	if (scrolled != NULL)
-		gtk_widget_hide (scrolled);
-
 	return dialog;
 }
 
@@ -668,113 +651,50 @@ _gtk_label_get_filename_text (GtkLabel   *label)
 	return text;
 }
 
-/* This function from gnome-panel/panel-util.c
- * (C) 1997, 1998, 1999, 2000 The Free Software Foundation
- * Copyright 2000 Helix Code, Inc.
- * Copyright 2000,2001 Eazel, Inc.
- * Copyright 2001 George Lebl
- * Copyright 2002 Sun Microsystems Inc.
- *
- * Authors: George Lebl
- *          Jacob Berkman
- *          Mark McLoughlin
- *
- * Modified by Paolo Bacchilega for the Quick Lounge applet
- */
-static char*
-panel_find_icon (GtkIconTheme  *icon_theme,
-		 const char    *icon_name,
-		 gint           size)
+
+GdkPixbuf *
+get_icon_pixbuf (GtkIconTheme *icon_theme,
+		 GThemedIcon  *icon,
+		 int           size)
 {
-	char        *retval  = NULL;
-	GtkIconInfo *icon_info = NULL;
-	char        *icon_no_extension;
-	char        *p;
+	char        **icon_names;
+	GtkIconInfo  *icon_info;
+	GdkPixbuf    *pixbuf;
+	GError       *error = NULL;
 
+	g_object_get (icon, "names", &icon_names, NULL);
 
-	if (icon_name == NULL || strcmp (icon_name, "") == 0)
-		return NULL;
+	icon_info = gtk_icon_theme_choose_icon (icon_theme, (const char **)icon_names, size, 0);
+	if (icon_info == NULL)
+		icon_info = gtk_icon_theme_lookup_icon (icon_theme, "text-x-generic", size, GTK_ICON_LOOKUP_USE_BUILTIN);
 
-	if (g_path_is_absolute (icon_name)) {
-		if (g_file_test (icon_name, G_FILE_TEST_EXISTS)) {
-			return g_strdup (icon_name);
-		} else {
-			char *basename;
-
-			basename = g_path_get_basename (icon_name);
-			retval = panel_find_icon (icon_theme,
-						  basename,
-						  size);
-			g_free (basename);
-
-			return retval;
-		}
+	pixbuf = gtk_icon_info_load_icon (icon_info, &error);
+	if (pixbuf == NULL) {
+		g_warning ("could not load icon pixbuf: %s\n", error->message);
+		g_clear_error (&error);
 	}
 
-	/* This is needed because some .desktop files have an icon name *and*
-	 * an extension as icon */
-	icon_no_extension = g_strdup (icon_name);
-	p = strrchr (icon_no_extension, '.');
-	if (p &&
-	    (strcmp (p, ".png") == 0 ||
-	     strcmp (p, ".xpm") == 0 ||
-	     strcmp (p, ".svg") == 0)) {
-	    *p = 0;
-	}
-
-	icon_info = gtk_icon_theme_lookup_icon (icon_theme,
-						icon_no_extension,
-						size,
-						0);
-	retval = g_strdup (gtk_icon_info_get_filename (icon_info));
-
-	g_free (icon_no_extension);
 	gtk_icon_info_free (icon_info);
+	g_strfreev (icon_names);
 
-	return retval;
+	return pixbuf;
 }
 
 
 GdkPixbuf *
-create_pixbuf (GtkIconTheme  *icon_theme,
-	       const char    *icon_name,
-	       int            icon_size)
+get_mime_type_pixbuf (const char   *mime_type,
+		      int           icon_size,
+		      GtkIconTheme *icon_theme)
 {
-	char      *icon_path;
-	GdkPixbuf *pixbuf;
-	int        iw;
-	int        ih;
+	GdkPixbuf *pixbuf = NULL;
+	GIcon     *icon;
 
-	icon_path = panel_find_icon (icon_theme, icon_name, icon_size);
-	if (icon_path == NULL)
-		return NULL;
+	if (icon_theme == NULL)
+		icon_theme = gtk_icon_theme_get_default ();
 
-	pixbuf = gdk_pixbuf_new_from_file (icon_path, NULL);
-	g_free (icon_path);
-
-	if (pixbuf == NULL)
-		return NULL;
-
-	iw = gdk_pixbuf_get_width (pixbuf);
-	ih = gdk_pixbuf_get_height (pixbuf);
-
-	if ((iw > icon_size) || (ih > icon_size)) {
-		GdkPixbuf *scaled;
-		gdouble    factor;
-		gdouble    max = icon_size;
-		int        new_w, new_h;
-
-		factor = MIN (max / iw, max / ih);
-		new_w  = MAX ((int) (iw * factor), 1);
-		new_h = MAX ((int) (ih * factor), 1);
-
-		scaled = gdk_pixbuf_scale_simple (pixbuf,
-						  new_w,
-						  new_h,
-						  GDK_INTERP_BILINEAR);
-		g_object_unref (pixbuf);
-		pixbuf = scaled;
-	}
+	icon = g_content_type_get_icon (mime_type);
+	pixbuf = get_icon_pixbuf (icon_theme, G_THEMED_ICON (icon), icon_size);
+	g_object_unref (icon);
 
 	return pixbuf;
 }
@@ -792,53 +712,45 @@ get_folder_pixbuf_size_for_list (GtkWidget *widget)
 }
 
 
+gboolean
+show_uri (GdkScreen   *screen,
+	  const char  *uri,
+	  guint32      timestamp,
+	  GError     **error)
+{
+	gboolean result;
+
+#ifndef GTK_2_13
+	GAppLaunchContext *app_context;
+
+	app_context = g_app_launch_context_new ();
+	result = g_app_info_launch_default_for_uri (uri, G_APP_LAUNCH_CONTEXT (app_context), error);
+
+	g_object_unref (app_context);
+#else
+	result = gtk_show_uri (screen, uri, timestamp, error);
+#endif
+
+	return result;
+}
+
+
 void
 show_help_dialog (GtkWindow  *parent,
-		  const char *link_id)
+		  const char *section)
 {
-	GError *err = NULL;
-	char *command;
-	const char *lang;
-	char *uri = NULL;
-	int i;
-	GdkScreen *gscreen;
+	char   *uri;
+	GError *error = NULL;
 
-	const char * const * langs = g_get_language_names ();
-
-	for (i = 0; langs[i]; i++) {
-		lang = langs[i];
-		if (strchr (lang, '.')) {
-			continue;
-		}
-
-		uri = g_build_filename(FR_DATADIR,
-				       "/gnome/help/file-roller/",
-					lang,
-				       "/file-roller.xml",
-					NULL);
-					
-		if (g_file_test (uri, G_FILE_TEST_EXISTS)) {
-                    break;
-		}
-	}
-	
-	if (link_id) {
-		command = g_strconcat ("gnome-open ghelp://", uri, "?", link_id, NULL);
-	} else {
-		command = g_strconcat ("gnome-open ghelp://", uri,  NULL);
-	}
-
-	gscreen = gdk_screen_get_default();
-	gdk_spawn_command_line_on_screen (gscreen, command, &err);
-
-	if (err != NULL) {
-		GtkWidget *dialog;
+	uri = g_strconcat ("ghelp:file-roller", section ? "?" : NULL, section, NULL);
+	if (! show_uri (gtk_window_get_screen (parent), uri, GDK_CURRENT_TIME, &error)) {
+  		GtkWidget *dialog;
 
 		dialog = _gtk_message_dialog_new (parent,
-						  GTK_DIALOG_DESTROY_WITH_PARENT, 
+						  GTK_DIALOG_DESTROY_WITH_PARENT,
 						  GTK_STOCK_DIALOG_ERROR,
 						  _("Could not display help"),
-						  err->message,
+						  error->message,
 						  GTK_STOCK_OK, GTK_RESPONSE_OK,
 						  NULL);
 		gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
@@ -851,6 +763,7 @@ show_help_dialog (GtkWindow  *parent,
 
 		gtk_widget_show (dialog);
 
-		g_error_free (err);
+		g_clear_error (&error);
 	}
+	g_free (uri);
 }
